<?php

namespace Drupal\block_class\Form;

use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\block\Entity\Block;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Form for Block Class Confirm Bulk Operation.
 */
class BlockClassConfirmBulkOperationForm extends ConfirmFormBase {

  /**
   * Bulk Operation type.
   *
   * @var string
   */
  protected $operation;

  /**
   * Classes to be added.
   *
   * @var string
   */
  protected $classesToBeAdded;

  /**
   * Current class.
   *
   * @var string
   */
  protected $currentClass;

  /**
   * New class.
   *
   * @var string
   */
  protected $newClass;

  /**
   * {@inheritdoc}
   */
  // @codingStandardsIgnoreLine
  public function buildForm(array $form, FormStateInterface $form_state, $operation = NULL, $classes_to_be_added = NULL, $current_class = NULL, $new_class = NULL) {

    // If the user doesn't have the sessions, restrict it.
    if (empty(\Drupal::service('session')->get('block_class_confirm_bulk_operation'))) {

      // Send a message to the user that is necessary fill the bulk operation
      // before running the operation.
      \Drupal::messenger()->addError($this->t('Is necessary fill the Bulk Operation form before running this operation'));

      // Redirect to Bulk Operations.
      \Drupal::service('block_class.helper')->redirectToBulkOperations();

    }

    // Get the default parameters.
    $this->operation = $operation;
    $this->classesToBeAdded = $classes_to_be_added;
    $this->currentClass = $current_class;
    $this->newClass = $new_class;

    // Get the session value.
    $block_class_confirm_bulk_operation = \Drupal::service('session')->get('block_class_confirm_bulk_operation');

    // Restrict this page to users with the session in the same of operation.
    if ($this->operation != $block_class_confirm_bulk_operation) {

      // Send a message to the user that is necessary fill the bulk operation
      // before running the operation.
      \Drupal::messenger()->addError($this->t('Is necessary fill the Bulk Operation form for <strong>@operation@</strong> before running this operation', [
        '@operation@' => $this->operation,
      ]));

      // Redirect to Bulk Operations.
      \Drupal::service('block_class.helper')->redirectToBulkOperations();

    }

    $message_to_confirm = (string) $this->getQuestion();

    $form['message_to_confirm'] = [
      '#type' => 'item',
      '#markup' => $message_to_confirm,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    // Load blocks. Todo: We'll implements DI here @codingStandardsIgnoreLine
    $blocks = Block::loadMultiple();

    foreach ($blocks as $block) {

      // If is insert include this class to all blocks.
      if ($this->operation == 'insert') {

        // Get the current block classes configured.
        $current_classes = $block->getThirdPartySetting('block_class', 'classes');

        // Add the new classes in the current ones.
        $current_classes = ' ' . $this->classesToBeAdded;

        // Using trim to remove spaces.
        $current_classes = trim($current_classes);

        // Store that in the Third Party Setting.
        $block->setThirdPartySetting('block_class', 'classes', $current_classes);

        // Save the block.
        $block->save();

        // Go to the next iteration.
        continue;

      }

      // If is update get all values to update.
      if ($this->operation == 'update') {

        // Get the current block classes configured.
        $current_block_classes = $block->getThirdPartySetting('block_class', 'classes');

        // If the current block class doesn't have this current class, skip.
        if (!preg_match("/\b" . $this->currentClass . "\b/i", $current_block_classes)) {
          return;
        }

        // Update the new block classes value with this replace.
        $new_block_classes = preg_replace("/\b" . $this->currentClass . "\b/i", $this->newClass, $current_block_classes);

        // Store that in the Third Party Setting.
        $block->setThirdPartySetting('block_class', 'classes', $new_block_classes);

        // Save the block.
        $block->save();

        // Go to the next iteration.
        continue;

      }

      if ($this->operation == 'delete') {

        // If there is ThirdPartySetting remove that.
        $block->unsetThirdPartySetting('block_class', 'classes');

        // Block save.
        $block->save();

      }
    }

    // Clear the session because the confirmation is done.
    \Drupal::service('session')->remove('block_class_confirm_bulk_operation');

    \Drupal::messenger()->addStatus($this->t('Bulk operation concluded'));

    // Get path bulk operation.
    $bulk_operation_path = Url::fromRoute('block_class.bulk_operations')->toString();

    // Get response.
    $response = new RedirectResponse($bulk_operation_path);

    // Send to confirmation.
    $response->send();
    exit;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() : string {
    return "block_class_confirm_bulk_operation_form";
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {

    return Url::fromRoute('block_class.bulk_operations');

  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {

    switch ($this->operation) {

      case 'insert':

        $message_to_confirm = $this->t('Do you want to insert class(es): <strong>@classes_to_be_added@</strong> to all blocks?', [
          '@classes_to_be_added@' => $this->classesToBeAdded,
        ]);

        break;

      case 'update':

        $message_to_confirm = $this->t('Do you want to update all block classes that have <strong>@current_class@</strong> to <strong>@new_class@</strong>?', [
          '@current_class@' => $this->currentClass,
          '@new_class@' => $this->newClass,
        ]);

        break;

      case 'delete':

        $message_to_confirm = $this->t('Do you want to run this bulk operation for all block classes?');

        break;
    }

    return $message_to_confirm;
  }

}
