<?php

namespace Drupal\block_class\Service;

use Drupal\Component\Uuid\UuidInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Path\PathMatcherInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\path_alias\AliasManagerInterface;
use Drupal\Core\Path\CurrentPathStack;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\Entity\EntityFormInterface;
use Drupal\block\Entity\Block;
use Drupal\Component\Utility\Html;
use Drupal\block_class\Constants\BlockClassConstants;

/**
 * Block Class Service Class.
 */
class BlockClassHelperService {

  use StringTranslationTrait;

  /**
   * The UUID service.
   *
   * @var \Drupal\Component\Uuid\UuidInterface
   */
  protected $uuidService;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Path Matcher.
   *
   * @var \Drupal\Core\Path\PathMatcherInterface
   */
  protected $pathMatcher;

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The user current.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The path alias manager.
   *
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected $aliasManager;

  /**
   * The current path.
   *
   * @var \Drupal\Core\Path\CurrentPathStack
   */
  protected $currentPath;

  /**
   * Drupal\Core\Entity\EntityRepositoryInterface service.
   *
   * @var \Drupal\Core\Entity\EntityRepositoryInterface
   */
  protected $entityRepository;

  /**
   * Construct of Block Class service.
   */
  public function __construct(LanguageManagerInterface $language_manager, EntityTypeManagerInterface $entity_manager, ConfigFactoryInterface $config_factory, Connection $database, RequestStack $request_stack, PathMatcherInterface $path_matcher, UuidInterface $uuid_service, AccountProxyInterface $current_user, AliasManagerInterface $alias_manager, CurrentPathStack $current_path, EntityRepositoryInterface $entityRepository) {
    $this->languageManager = $language_manager;
    $this->entityTypeManager = $entity_manager;
    $this->pathMatcher = $path_matcher;
    $this->request = $request_stack->getCurrentRequest();
    $this->configFactory = $config_factory;
    $this->database = $database;
    $this->uuidService = $uuid_service;
    $this->currentUser = $current_user;
    $this->aliasManager = $alias_manager;
    $this->currentPath = $current_path;
    $this->entityRepository = $entityRepository;
  }

  /**
   * Method to redirect to Bulk Operations.
   */
  public function redirectToBulkOperations() {

    // Get path bulk operation.
    $bulk_operation_path = Url::fromRoute('block_class.bulk_operations')->toString();

    // Get response.
    $response = new RedirectResponse($bulk_operation_path);

    // Send to confirmation.
    $response->send();
    exit;

  }

  /**
   * Method to do a form alter.
   */
  public function blockClassFormAlter(&$form, &$form_state) {

    // If the user don't have permission, skip that.
    if (!\Drupal::currentUser()->hasPermission(BlockClassConstants::BLOCK_CLASS_PERMISSION)) {
      return;
    }

    $form_object = $form_state->getFormObject();

    if ($form_object instanceof EntityFormInterface) {

      /** @var \Drupal\block\BlockInterface $block */
      $block = $form_object->getEntity();

      // This will automatically be saved in the third party settings.
      $form['third_party_settings']['#tree'] = TRUE;

      // Default field type.
      $field_type = 'textfield';

      // Default value for maxlength.
      $maxlength_block_class_field = 255;

      // Get config object.
      $config = \Drupal::config('block_class.settings');

      // Get the field type if exists.
      if (!empty($config->get('field_type'))) {
        $field_type = $config->get('field_type');
      }

      // Get maxlength if exists.
      if (!empty($config->get('maxlength_block_class_field'))) {
        $maxlength_block_class_field = $config->get('maxlength_block_class_field');
      }

      $form['third_party_settings']['block_class']['classes'] = [
        '#type' => $field_type,
        '#title' => $this->t('CSS class(es)'),
        '#description' => $this->t('Customize the styling of this block by adding CSS classes. Separate multiple classes by spaces. The maxlength configured is @maxlength_block_class_field@. If necessary you can update it <a href="/admin/config/content/block-class/settings">here</a>', [
          '@maxlength_block_class_field@' => $maxlength_block_class_field,
        ]),
        '#default_value' => $block->getThirdPartySetting('block_class', 'classes'),
        '#maxlength' => $maxlength_block_class_field,
      ];
    }
  }

  /**
   * Method to do the preprocess block.
   */
  public function blockClassPreprocessBlock(&$variables) {

    // Blocks coming from page manager widget does not have id. If there is no
    // Block ID, skip that.
    if (empty($variables['elements']['#id'])) {
      return;
    }

    // Load the block by ID.
    $block = Block::load($variables['elements']['#id']);

    // Verify if the current block has Third Party Settings with classes.
    if ($block && $classes = $block->getThirdPartySetting('block_class', 'classes')) {

      // Get all classes if exists.
      $classes_array = explode(' ', $classes);

      // Add all classes.
      foreach ($classes_array as $class) {
        $variables['attributes']['class'][] = Html::cleanCssIdentifier($class, []);
      }
    }
  }

}
